//!
//!
use super::{models, schema::banks};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of banks").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_member: &models::NewBank,
  ) -> Result<models::Bank, Error> {
    let inserted = diesel::insert_into(banks::table)
      .values(new_member)
      .get_result::<models::Bank>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::Bank>, Error> {
    match banks::table.find(id).first::<models::Bank>(conn) {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = banks::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.name_like {
        q = q.filter(banks::dsl::name.like(sp));
      }
      if let Some(sp) = s.show {
        q = q.filter(banks::dsl::show.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(banks::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(banks::dsl::memo.like(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::Bank>, Error> {
    let mut q = banks::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.name_like {
        q = q.filter(banks::dsl::name.like(sp));
      }
      if let Some(sp) = s.show {
        q = q.filter(banks::dsl::show.eq(sp));
      }
      if let Some(sp) = s.can_use {
        q = q.filter(banks::dsl::can_use.eq(sp));
      }
      if let Some(sp) = &s.memo_like {
        q = q.filter(banks::dsl::memo.like(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(banks::name.asc());
            }
            "created_at" => {
              q = q.order_by(banks::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(banks::updated_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(banks::name.desc());
            }
            "created_at" => {
              q = q.order_by(banks::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(banks::updated_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::Bank>(conn)
  }

  ///
  pub fn update(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
    modify: &models::ModifyBank,
  ) -> Result<u64, Error> {
    use banks::dsl;

    diesel::update(dsl::banks.filter(dsl::id.eq(id)))
      .set(modify)
      .execute(conn)
      .map(|c| c as u64)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: uuid::Uuid) -> Result<u64, Error> {
    use banks::dsl;

    diesel::delete(banks::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
